declare namespace GitHubAPI {
    /** Not actually part of the GitHub API */
    interface RepoReference {
        owner: string;
        name: string;
    }
    interface Repository {
        id: number;
        owner: User;
        name: string;
        full_name: string;
        description: string;
        private: boolean;
        fork: boolean;
        url: string;
    }
    type ReactionType = "+1" | "-1" | "laugh" | "confused" | "heart" | "hooray";
    interface Reaction {
        id: number;
        user: User;
        content: ReactionType;
        created_at: string;
    }
    interface Issue {
        url: string;
        repository_url: string;
        labels_url: string;
        comments_url: string;
        events_url: string;
        html_url: string;
        id: number;
        number: number;
        title: string;
        body: string;
        user: User;
        labels: Label[] | undefined;
        state: "open" | "closed";
        locked: boolean;
        assignee: User | null;
        assignees: User[];
        milestone: Milestone | null;
        comments: number;
        created_at: string;
        updated_at: string;
        closed_at: string | null;
        repository?: Repository;
        pull_request?: {
            url: string;
            html_url: string;
            diff_url: string;
            patch_url: string;
        };
    }
    /**
     * When the GitHub API lists PRs, it includes some (but not all) PR-specific fields.
     */
    interface PullRequestFromList extends Issue {
        merged_at: string | null;
        merge_commit_sha: string;
        url: string;
        repository_url: string;
        labels_url: string;
        comments_url: string;
        events_url: string;
        html_url: string;
        id: number;
        number: number;
        title: string;
        body: string;
        user: User;
        state: "open" | "closed";
        locked: boolean;
        assignee: User | null;
        assignees: User[];
        milestone: Milestone | null;
        comments: number;
        created_at: string;
        updated_at: string;
        closed_at: string | null;
        repository?: Repository;
        pull_request?: {
            url: string;
            html_url: string;
            diff_url: string;
            patch_url: string;
        };
    }
    interface PullRequestReview {
        id: number;
        user: User;
        body: string;
        commit_id: string;
        state: "APPROVED" | "CHANGES_REQUESTED" | "COMMENTED";
        submitted_at: string;
        html_url: string;
        pull_request_url: string;
    }
    /**
     * dirty: Merge conflict. Merging will be blocked
     * unknown: Mergeability was not checked yet. Merging will be blocked
     * blocked: Blocked by a failing/missing required status check.
     * behind: Head branch is behind the base branch. Only if required status checks is enabled but loose policy is not. Merging will be blocked.
     * unstable: Failing/pending commit status that is not part of the required status checks. Merging is allowed (yellow box).
     * has_hooks: This is for Enterprise only, if a repo has custom pre-receive hooks. Merging is allowed (green box).
     * clean: No conflicts, everything good. Merging is allowed (green box).
     */
    type MergeableState = "dirty" | "unknown" | "blocked" | "behind" | "unstable" | "has_hooks" | "clean";
    interface PullRequest extends PullRequestFromList {
        merged: boolean;
        /** Mergeable is null if it's not computed yet */
        mergeable: boolean | null;
        /** Allegedly undocumented but has been around for 5 years */
        mergeable_state: MergeableState;
        merged_by?: User;
        comments: number;
        commits: number;
        additions: number;
        deletions: number;
        changed_files: number;
        maintainer_can_modify: boolean;
        statuses_url: string;
        head: Commit;
        base: Commit;
        review_comments: number;
    }
    interface PullRequestFile {
        sha: string;
        filename: string;
        status: "added" | "modified" | "removed";
        additions: number;
        deletions: number;
        changes: number;
    }
    interface PullRequestCommit {
        url: string;
        sha: string;
        commit: {
            author: Committer;
            committer: Committer;
        };
        author: User;
        committer: User;
        parents: {
            url: string;
            sha: string;
        }[];
    }
    interface Committer {
        name: string;
        email: string;
        date: string;
    }
    interface Commit {
        label: string;
        ref: string;
        sha: string;
        user: User;
        repo: Repository;
    }
    type StatusSummary = "success" | "pending" | "failure" | "error";
    interface CombinedStatus {
        state: StatusSummary;
        sha: string;
        total_count: number;
    }
    interface CommitStatusesElement {
        url: string;
        id: number;
        state: StatusSummary;
        description: string;
        target_url: string;
        context: string;
        created_at: string;
        updated_at: string;
        creator: User;
    }
    interface CommitStatus {
    }
    interface User {
        login: string;
        id: number;
        avatar_url: string;
        gravatar_id: string;
        url: string;
        html_url: string;
        followers_url: string;
        following_url: string;
        gists_url: string;
        starred_url: string;
        subscriptions_url: string;
        organizations_url: string;
        repos_url: string;
        events_url: string;
        received_events_url: string;
        type: "User";
        site_admin: boolean;
    }
    interface Label {
        url: string;
        name: string;
        color: string;
    }
    interface Milestone {
        url: string;
        html_url: string;
        labels_url: string;
        id: number;
        number: number;
        title: string;
        description: string;
        creator: User;
        open_issues: number;
        closed_issues: number;
        state: "open" | "closed";
        created_at: string;
        updated_at: string;
        due_on: string | null;
        closed_at: string | null;
    }
    type IssueEvent = IssueClosedEvent | IssueReopenedEvent | IssueSubscribedEvent | IssueMergedEvent | IssueReferencedEvent | IssueMentionedEvent | IssueAssignedEvent | IssueUnassignedEvent | IssueLabeledEvent | IssueUnlabeledEvent | IssueMilestonedEvent | IssueDemilestonedEvent | IssueRenamedEvent | IssueLockedEvent | IssueUnlockedEvent | IssueHeadRefDeletedEvent | IssueHeadRefRestoredEvent;
    interface IssueEventBase {
        id: number;
        url: string;
        actor: User | null;
        commit_id: string | null;
        commit_url: string | null;
        created_at: string;
    }
    interface IssueClosedEvent extends IssueEventBase {
        event: "closed";
    }
    interface IssueReopenedEvent extends IssueEventBase {
        event: "reopened";
    }
    interface IssueSubscribedEvent extends IssueEventBase {
        event: "subscribed";
    }
    interface IssueMergedEvent extends IssueEventBase {
        event: "merged";
    }
    interface IssueReferencedEvent extends IssueEventBase {
        event: "referenced";
    }
    interface IssueMentionedEvent extends IssueEventBase {
        event: "mentioned";
    }
    interface IssueLabeledEvent extends IssueEventBase {
        event: "labeled";
        label: {
            name: string;
            color: string;
        };
    }
    interface IssueAssignedEvent extends IssueEventBase {
        event: "assigned";
        assigner: User;
        asignee: User;
    }
    interface IssueUnassignedEvent extends IssueEventBase {
        event: "unassigned";
        assigner: User;
        asignee: User;
    }
    interface IssueUnlabeledEvent extends IssueEventBase {
        event: "unlabeled";
        label: {
            name: string;
            color: string;
        };
    }
    interface IssueMilestonedEvent extends IssueEventBase {
        event: "milestoned";
        milestone: {
            title: string;
        };
    }
    interface IssueDemilestonedEvent extends IssueEventBase {
        event: "demilestoned";
        milestone: {
            title: string;
        };
    }
    interface IssueRenamedEvent extends IssueEventBase {
        event: "renamed";
        from: string;
        to: string;
    }
    interface IssueLockedEvent extends IssueEventBase {
        event: "locked";
    }
    interface IssueUnlockedEvent extends IssueEventBase {
        event: "unlocked";
    }
    interface IssueHeadRefDeletedEvent extends IssueEventBase {
        event: "head_ref_deleted";
    }
    interface IssueHeadRefRestoredEvent extends IssueEventBase {
        event: "head_ref_restored";
    }
    interface IssueComment {
        url: string;
        html_url: string;
        issue_url: string;
        id: number;
        user: User;
        created_at: string;
        updated_at: string;
        body: string;
    }
    interface Project {
        id: number;
        name: string;
        body: string;
        number: number;
        state: string;
    }
    interface ProjectColumn {
        id: number;
        name: string;
    }
    interface ProjectColumnCard {
        id: number;
        note: string | null;
        content_url: string;
    }
}
