/// <reference types="moment" />
import moment = require('moment');
import { Cache } from './cache';
import * as Actions from './action';
export declare function useCache(c: Cache): void;
export declare function parseBasicRepoReference(ownerSlashName: string): Repository;
export declare function parseRepoReferenceFromURL(url: string): Repository;
export declare class User {
    private originalData;
    private static pool;
    static fromLogin(login: string): Promise<User>;
    static fromData(data: GitHubAPI.User): User;
    readonly login: string;
    static getCacheKey(login: string): string;
    getCacheKey(): string;
    private constructor();
    update(data: GitHubAPI.User): void;
}
export declare class Label {
    repo: GitHubAPI.RepoReference;
    private static pool;
    static fromData(repo: GitHubAPI.RepoReference, data: GitHubAPI.Label): Label;
    readonly name: string;
    readonly color: string;
    private constructor();
    update(data: [GitHubAPI.RepoReference, GitHubAPI.Label]): void;
}
export declare class IssueComment {
    private originalData;
    issue: Issue;
    readonly id: number;
    readonly user: User;
    readonly created_at: moment.Moment;
    readonly updated_at: moment.Moment;
    readonly body: string;
    readonly repository: Repository;
    static fromData(originalData: GitHubAPI.IssueComment, issue: Issue): IssueComment;
    private constructor();
    update(data: GitHubAPI.IssueComment): void;
    getReactions(): Promise<GitHubAPI.Reaction[]>;
}
export declare class Milestone {
    private originalData;
    /** Internal milestone id */
    readonly id: number;
    /** This is the actual user-facing number */
    readonly number: number;
    /** Title  */
    readonly title: string;
    /** Description  */
    readonly description: string;
    /** State can be "open" or "closed" */
    readonly state: "open" | "closed";
    /** Who created this milestone */
    readonly creator: User;
    /** When this milestone was created */
    readonly created_at: moment.Moment;
    /** When this milestone was last updated */
    readonly updated_at: moment.Moment;
    /** When this milestone is due, if applicable */
    readonly due_on: moment.Moment | null;
    /** When this milestone was closed, if applicable */
    readonly closed_at: moment.Moment | null;
    constructor(originalData: GitHubAPI.Milestone);
}
export declare class Repository {
    readonly owner: string;
    readonly name: string;
    private static cache;
    static create(owner: string, name: string): Repository;
    readonly reference: GitHubAPI.RepoReference;
    private constructor();
}
export declare abstract class IssueOrPullRequest {
    private originalData;
    static fromData(data: GitHubAPI.Issue): Promise<Issue | PullRequest>;
    static getCacheKey(repo: GitHubAPI.RepoReference, number: number, isPR: boolean): string;
    static getCommentsCacheKey(repo: GitHubAPI.RepoReference, number: number, isPR: boolean): string;
    static getCacheKeyBasePath(repo: GitHubAPI.RepoReference, number: number, isPR: boolean): string;
    /** This is the actual user-facing number */
    readonly number: number;
    /** An internal GitHub ID number */
    readonly id: number;
    /** Title */
    readonly title: string;
    /** The main body of the issue */
    readonly body: string;
    /** State can be "open" or "closed" */
    readonly state: "open" | "closed";
    /** Whether the issue has been locked or not */
    readonly locked: boolean;
    readonly html_url: string;
    /** When this issue was created */
    readonly created_at: moment.Moment;
    /** When this issue was last updated */
    readonly updated_at: moment.Moment;
    /** When this issue was closed, if it's closed */
    readonly closed_at: moment.Moment | null;
    /** The labels this issue has */
    readonly labels: ReadonlyArray<Label>;
    /** The author of this issue */
    readonly user: User;
    /** A list, possibly empty, of who is assigned to this issue */
    readonly assignees: ReadonlyArray<User>;
    /** A milestone, if one is assigned */
    readonly milestone: Milestone | null;
    readonly abstract isPullRequest: boolean;
    readonly repository: Repository;
    protected constructor(originalData: GitHubAPI.Issue);
    /** Returns a string like 'Microsoft/TypeScript#14' */
    readonly fullName: string;
    update(): void;
    getComments(): Promise<IssueComment[]>;
    /**
    * Adds a label to an issue or PR.
    */
    addLabel(...labels: string[]): Actions.IAction;
    /**
     * Adds labels to an issue or PR.
     */
    addLabels(labels: string[]): Actions.IAction;
    /**
     * Deletes a label from an issue.
     */
    removeLabel(...labels: string[]): Actions.IAction;
    /**
     * Deletes labels from an issue.
     */
    removeLabels(labels: string[]): Actions.IAction;
    /**
     * Sets, exactly, which labels are on an issue
     */
    setLabels(...labels: string[]): Actions.IAction;
    /**
     * Shortcut method for setting/clearing labels with an object literal, e.g.
     * issue.setHasLabels({
     *  "Needs Info": false,
     *  "Ready for Triage": true
     * }).
     * You can specify a null value to cause nothing to happen either way
     */
    setHasLabels(labelMap: {
        [key: string]: boolean | null;
    }): void;
    /**
     * Returns true if this issue has the specified label
     */
    hasLabel(labelName: string | Label): boolean;
    /**
     * Adds or updates a comment with the specified slug to the issue
     */
    addComment(slug: string, body: string): Actions.IAction;
    lock(): Actions.IAction;
    unlock(): Actions.IAction;
    close(): Actions.IAction;
    reopen(): Actions.IAction;
}
export declare class Issue extends IssueOrPullRequest {
    static fromData(data: GitHubAPI.Issue): never;
    static fromIssueData(data: GitHubAPI.Issue): Issue;
    readonly isPullRequest: boolean;
    private constructor();
}
export declare type StatusSummary = "pass" | "fail" | "pending";
export declare class PullRequest extends IssueOrPullRequest {
    static fromReference(repo: GitHubAPI.RepoReference, number: number | string): Promise<PullRequest>;
    static fromData(data: GitHubAPI.Issue): never;
    static getPRCacheKey(repo: GitHubAPI.RepoReference, number: number, isPR: boolean): string;
    static fromIssueAndPRData(prData: GitHubAPI.PullRequest, issueData: GitHubAPI.Issue): Promise<PullRequest>;
    static fromPullRequestData(prData: GitHubAPI.PullRequest, issueData: GitHubAPI.Issue): PullRequest;
    head: GitHubAPI.Commit;
    merge_commit_sha: string;
    merged: boolean;
    mergeable: boolean | null;
    mergeable_state: GitHubAPI.MergeableState;
    comments: number;
    commits: number;
    additions: number;
    deletions: number;
    changed_files: number;
    readonly isPullRequest: boolean;
    private constructor();
    getStatusSummary(): Promise<GitHubAPI.StatusSummary>;
    getStatus(): Promise<GitHubAPI.CombinedStatus>;
    getReviews(): Promise<GitHubAPI.PullRequestReview[]>;
    getCommitsRaw(): Promise<GitHubAPI.PullRequestCommit[]>;
    getFilesRaw(): Promise<GitHubAPI.PullRequestFile[]>;
    getMergeableState(): Promise<boolean | null>;
}
export declare class Project {
    projectId: number;
    columns: ProjectColumn[];
    static create(projectId: number): Promise<Project>;
    private constructor();
    setIssueColumn(issue: IssueOrPullRequest, targetColumn: ProjectColumn | undefined): Actions.IAction;
    doSetIssueColumn(issue: IssueOrPullRequest, targetColumn: ProjectColumn | undefined): Promise<void>;
    findColumnByName(name: string | RegExp): ProjectColumn | undefined;
}
export declare class ProjectColumn {
    columnId: number;
    name: string;
    static create(columnId: number, name: string): Promise<ProjectColumn>;
    cards: ProjectCard[];
    private constructor();
    findProjectCardForIssue(issue: IssueOrPullRequest): ProjectCard | undefined;
}
export declare class ProjectCard {
    private data;
    id: number;
    constructor(data: GitHubAPI.ProjectColumnCard);
    getIssueNumber(): number | undefined;
}
